@@ -55,31 +55,27 @@ Regexp.
 
 =cut
 
+my @Valid_Users;
+
 sub ALLOW_LOG_CHANGE {
     my @args = @_;
 
-    my $conf = $SVN::Hooks::Confs->{$HOOK};
-
     foreach my $who (@args) {
 	if (not ref $who or ref $who eq 'Regexp') {
-	    push @{$conf->{users}}, $who;
+	    push @Valid_Users, $who;
 	}
 	else {
 	    croak "$HOOK: invalid argument '$who'\n";
 	}
     }
 
-    $conf->{'pre-revprop-change'} = \&pre_revprop_change;
+    PRE_REVPROP_CHANGE(\&pre_revprop_change);
 
     return 1;
 }
 
-$SVN::Hooks::Inits{$HOOK} = sub {
-    return { users => [] };
-};
-
 sub pre_revprop_change {
-    my ($self, $svnlook, $rev, $author, $propname, $action) = @_;
+    my ($svnlook, $rev, $author, $propname, $action) = @_;
 
     $propname eq 'svn:log'
 	or croak "$HOOK: the revision property $propname cannot be changed.\n";
@@ -88,9 +84,9 @@ sub pre_revprop_change {
 	or croak "$HOOK: a revision log can only be modified, not added or deleted.\n";
 
     # If no users are specified, anyone can do it.
-    return unless @{$self->{users}};
+    return unless @Valid_Users;
 
-    for my $user (@{$self->{users}}) {
+    for my $user (@Valid_Users) {
 	return if not ref $user and $author eq $user or $author =~ $user;
     }
 
@@ -140,7 +136,7 @@ L<http://search.cpan.org/dist/SVN-Hooks>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2008-2009 CPqD, all rights reserved.
+Copyright 2008-2011 CPqD, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
