@@ -29,7 +29,7 @@ It's active in the C<pre-commit> and/or the C<post-commit> hook.
 
 It's configured by the following directives.
 
-=head2 CHECK_JIRA_CONFIG(BASEURL, LOGIN, PASSWORD, [REGEXP])
+=head2 CHECK_JIRA_CONFIG(BASEURL, LOGIN, PASSWORD [, REGEXP [, REGEXP]])
 
 This directive specifies how to connect and to authenticate to the
 JIRA server. BASEURL is the base URL of the JIRA server, usually,
@@ -37,15 +37,22 @@ something like C<http://jira.example.com/jira>. LOGIN and PASSWORD are
 the credentials of a JIRA user who has browsing rights to the JIRA
 projects that will be referenced in the commit logs.
 
-The fourth argument is optional. It must be a qr/Regexp/ object that
-will be used to match against the commit logs in order to extract the
-list of JIRA issue keys. By default, the JIRA keys are looked for in
-the whole commit log. Sometimes this can be suboptimal because the
-user can introduce in the message some text that inadvertently looks
-like a JIRA issue key whithout being so. With this argument, the log
-message is matched against the REGEXP and only the first matched group
-(i.e., the part of the message captured by the first parenthesis
-(C<$1>)) is used to look for JIRA issue keys.
+The fourth argument is an optional qr/Regexp/ object. It will be used
+to match against the commit logs in order to extract the list of JIRA
+issue keys. By default, the JIRA keys are looked for in the whole
+commit log, which is equivalent to qr/(.*)/. Sometimes this can be
+suboptimal because the user can introduce in the message some text
+that inadvertently looks like a JIRA issue key whithout being so. With
+this argument, the log message is matched against the REGEXP and only
+the first matched group (i.e., the part of the message captured by the
+first parenthesis (C<$1>)) is used to look for JIRA issue keys.
+
+The fifth argument is another optional qr/Regexp/ object. It is used
+to match JIRA project keys, which match qr/[A-Z]{2,}/ by
+default. However, since you can specify different patterns for JIRA
+project keys
+(L<http://confluence.atlassian.com/display/JIRA/Configuring+Project+Keys>),
+you need to be able to specify this here too.
 
 The JIRA issue keys are extracted from the commit log (or the part of
 it specified by the REGEXP) with the following pattern:
@@ -53,27 +60,37 @@ C<qr/\b([A-Z]+-\d+)\b/g>;
 
 =cut
 
+my ($BaseURL, $Login, $Passwd, $MatchLog, $MatchKey);
+my $JIRA;
+my @Checks;
+my %Defaults = (
+    require     => 1,
+    valid       => 1,
+    unresolved  => 1,
+    by_assignee => 0,
+);
+
 sub CHECK_JIRA_CONFIG {
-    my ($baseURL, $login, $passwd, $match) = @_;
+    ($BaseURL, $Login, $Passwd, $MatchLog, $MatchKey) = @_;
 
-    if (@_ == 3) {
-	$match = qr/(.*)/;
+    if (defined $MatchKey) {
+	ref $MatchKey && ref $MatchKey eq 'Regexp'
+	    or croak "CHECK_JIRA_CONFIG: fifth argument must be a Regexp.\n";
+    } else {
+	$MatchKey = qr/[A-Z]{2,}/;
     }
-    elsif (@_ == 4) {
-	ref $match eq 'Regexp'
+
+    if (defined $MatchLog) {
+	ref $MatchLog && ref $MatchLog eq 'Regexp'
 	    or croak "CHECK_JIRA_CONFIG: fourth argument must be a Regexp.\n";
-    }
-    else {
-	croak "CHECK_JIRA_CONFIG: requires three or four arguments.\n";
+    } else {
+	$MatchLog = qr/[A-Z]{2,}/;
     }
 
-    $baseURL =~ s:/+$::;
+    @_ >= 3 && @_ <= 5
+	or croak "CHECK_JIRA_CONFIG: requires three, four, or five arguments.\n";
 
-    my $conf = $SVN::Hooks::Confs->{$HOOK};
-    $conf->{conf} = {
-	conf  => [$baseURL, $login, $passwd],
-	match => $match,
-    };
+    $BaseURL =~ s/\/+$//;
 
     return 1;
 }
@@ -126,13 +143,23 @@ reference issues to which she is the current assignee.
 
 If the above checks aren't enough you can pass a code reference
 (subroutine) to this option. The subroutine will be called once for
-each referenced issue with two arguments: the JIRA::Client object used
-to talk to the JIRA server and a reference to a RemoteIssue
-object. The subroutine must simply return with no value to indicate
-success and must die to indicate failure.
+each referenced issue with three arguments:
 
-Plese, read the JIRA::Client module documentation to understand how to
-use these objects.
+=over
+
+=item the JIRA::Client object used to talk to the JIRA server.
+
+=item the RemoteIssue object representing the issue.
+
+=item the SVN::Look object used to grok information about the commit.
+
+=back
+
+The subroutine must simply return with no value to indicate success
+and must die to indicate failure.
+
+Plese, read the JIRA::Client and SVN::Look modules documentation to
+understand how to use these objects.
 
 =item check_all => CODE-REF
 
@@ -141,8 +168,16 @@ to check some relation among all the referenced issues. In this case,
 pass a code reference to this option. It will be called once for the
 commit. Its first argument is the JIRA::Client object used to talk to
 the JIRA server. The following arguments are references to RemoteIssue
-objects for every referenced issue. The subroutine must simply return
-with no value to indicate success and must die to indicate failure.
+objects for every referenced issue. The last argument is the SVN::Look
+object used to grok information about the commit. The subroutine must
+simply return with no value to indicate success and must die to
+indicate failure.
+
+=item check_all_svnlook => CODE-REF
+
+This check is the same as the previous one, except that the first
+argument passed to the routine is the SVN::Look object used to grok
+information about the commit. The rest of the arguments are the same.
 
 =item post_action => CODE-REF
 
@@ -171,7 +206,7 @@ qr/Regexp/.
     # JIRA issues that have at least one fix version.
 
     sub is_scheduled {
-        my ($jira, $issue) = @_;
+        my ($jira, $issue, $svnlook) = @_;
         return scalar @{$issue->{fixVersions}};
     }
     CHECK_JIRA(qr/^(trunk|branches/fix)/ => {
@@ -238,14 +273,15 @@ sub _validate_code {
 }
 
 my %opt_checks = (
-    projects    => \&_validate_projects,
-    require     => \&_validate_bool,
-    valid       => \&_validate_bool,
-    unresolved  => \&_validate_bool,
-    by_assignee => \&_validate_bool,
-    check_one   => \&_validate_code,
-    check_all   => \&_validate_code,
-    post_action => \&_validate_code,
+    projects          => \&_validate_projects,
+    require           => \&_validate_bool,
+    valid             => \&_validate_bool,
+    unresolved        => \&_validate_bool,
+    by_assignee       => \&_validate_bool,
+    check_one         => \&_validate_code,
+    check_all         => \&_validate_code,
+    check_all_svnlook => \&_validate_code,
+    post_action       => \&_validate_code,
 );
 
 sub CHECK_JIRA {
@@ -261,42 +297,27 @@ sub CHECK_JIRA {
 	$opts->{$opt} = $opt_checks{$opt}->($opt, $opts->{$opt});
     }
 
-    my $conf = $SVN::Hooks::Confs->{$HOOK};
     if (ref $regex) {
-	push @{$conf->{checks}}, [$regex => $opts];
+	push @Checks, [$regex => $opts];
     }
     else {
 	while (my ($opt, $val) = each %$opts) {
-	    $conf->{defaults}{$opt} = $val;
+	    $Defaults{$opt} = $val;
 	}
     }
-    $conf->{'pre-commit'} = \&pre_commit;
-    $conf->{'post-commit'} = \&post_commit if exists $opts->{post_action};
+    PRE_COMMIT(\&pre_commit);
+    POST_COMMIT(\&post_commit) if exists $opts->{post_action};
 
     return 1;
 }
 
-$SVN::Hooks::Inits{$HOOK} = sub {
-    return {
-	checks   => [],
-	defaults => {
-	    require     => 1,
-	    valid       => 1,
-	    unresolved  => 1,
-	    by_assignee => 0,
-	},
-    };
-};
-
 sub _pre_checks {
-    my ($self, $svnlook, $keys, $opts) = @_;
-
-    my $conf = $self->{conf};
+    my ($svnlook, $keys, $opts) = @_;
 
     # Grok and check each JIRA issue
     my @issues;
     foreach my $key (@$keys) {
-	my $issue = eval {$conf->{jira}->getIssue($key)};
+	my $issue = eval {$JIRA->getIssue($key)};
 	if ($opts->{valid}) {
 	    croak "$HOOK: issue $key is not valid: $@\n" if $@;
 	}
@@ -311,47 +332,51 @@ sub _pre_checks {
 		if $author ne $issue->{assignee};
 	}
 	if (my $check = $opts->{check_one}) {
-	    $check->($conf->{jira}, $issue);
+	    $check->($JIRA, $issue, $svnlook);
 	}
 	push @issues, $issue;
     }
 
     if (my $check = $opts->{check_all}) {
-	$check->($conf->{jira}, @issues) if @issues;
+	$check->($JIRA, @issues) if @issues;
+    }
+
+    if (my $check = $opts->{check_all_svnlook}) {
+	$check->($svnlook, $JIRA, @issues) if @issues;
     }
 
     return;
 }
 
 sub _post_action {
-    my ($self, $svnlook, $keys, $opts) = @_;
+    my ($svnlook, $keys, $opts) = @_;
 
     if (my $action = $opts->{post_action}) {
-	$action->($self->{conf}->{jira}, $svnlook, @$keys);
+	$action->($JIRA, $svnlook, @$keys);
     }
 
     return;
 }
 
 sub _check_if_needed {
-    my ($self, $svnlook, $docheck) = @_;
+    my ($svnlook, $docheck) = @_;
 
-    my $conf = $self->{conf}
+    defined $BaseURL
 	or croak "$HOOK: plugin not configured. Please, use the CHECK_JIRA_CONFIG directive.\n";
 
     my @files = $svnlook->changed();
 
-    foreach my $check (@{$self->{checks}}) {
+    foreach my $check (@Checks) {
 	my ($regex, $opts) = @$check;
 
 	for my $file (@files) {
 	    if ($file =~ $regex) {
 
 		# Grok the JIRA issue keys from the commit log
-		my ($match) = ($svnlook->log_msg() =~ $conf->{match});
-		my @keys    = defined $match ? $match =~ /\b[A-Z]+-\d+\b/g : ();
+		my ($match) = ($svnlook->log_msg() =~ $MatchLog);
+		my @keys    = defined $match ? $match =~ /\b$MatchKey-\d+\b/g : ();
 
-		my %opts = (%{$self->{defaults}}, %$opts);
+		my %opts = (%Defaults, %$opts);
 
 		if ($opts{require}) {
 		    croak "$HOOK: you must cite at least one JIRA issue key in the commit message.\n"
@@ -360,13 +385,22 @@ sub _check_if_needed {
 
 		return unless @keys;
 
+		# Check if there is a restriction on the project keys allowed
+		if (exists $opts->{projects}) {
+		    foreach my $key (@keys) {
+			my ($pkey, $pnum) = split /-/, $key;
+			croak "$HOOK: issue $key is not allowed. You must cite only JIRA issues for the following projects: ", join(', ', sort keys %{$opts->{projects}}), ".\n"
+			    unless exists $opts->{projects}{$pkey};
+		    }
+		}
+
 		# Connect to JIRA if not yet connected.
-		unless (exists $conf->{jira}) {
-		    $conf->{jira} = eval {JIRA::Client->new(@{$conf->{conf}})};
+		unless (defined $JIRA) {
+		    $JIRA = eval {JIRA::Client->new($BaseURL, $Login, $Passwd)};
 		    croak "CHECK_JIRA_CONFIG: cannot connect to the JIRA server: $@\n" if $@;
 		}
 
-		$docheck->($self, $svnlook, \@keys, \%opts);
+		$docheck->($svnlook, \@keys, \%opts);
 		last;
 	    }
 	}
@@ -376,14 +410,14 @@ sub _check_if_needed {
 }
 
 sub pre_commit {
-    my ($self, $svnlook) = @_;
-    _check_if_needed($self, $svnlook, \&_pre_checks);
+    my ($svnlook) = @_;
+    _check_if_needed($svnlook, \&_pre_checks);
     return;
 }
 
 sub post_commit {
-    my ($self, $svnlook) = @_;
-    _check_if_needed($self, $svnlook, \&_post_action);
+    my ($svnlook) = @_;
+    _check_if_needed($svnlook, \&_post_action);
     return;
 }
 
@@ -430,7 +464,7 @@ L<http://search.cpan.org/dist/SVN-Hooks>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009 CPqD, all rights reserved.
+Copyright 2009-2011 CPqD, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
